@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -24,7 +24,6 @@ from pickle import dump, load
 from urllib.parse import urlparse
 from time import time
 from getpass import getuser
-import json
 from signal import signal, SIGINT, SIGTERM
 
 from eolie.settings import Settings
@@ -199,33 +198,6 @@ class Application(Gtk.Application):
         if uri in self.__pinned:
             self.__pinned.remove(uri)
 
-    def set_profiles(self):
-        """
-            Set profiles
-        """
-        try:
-            f = Gio.File.new_for_path(EOLIE_DATA_PATH + "/profiles.json")
-            if f.query_exists():
-                (status, contents, tag) = f.load_contents(None)
-                self.__profiles = json.loads(contents.decode("utf-8"))
-            else:
-                PROFILES = {"default": _("Default"),
-                            "social": _("Social networks"),
-                            "work": _("Work"),
-                            "shopping": _("Shopping"),
-                            "personal": _("Personal"),
-                            "finance": _("Finance"),
-                            "sport": _("Sport")}
-                content = json.dumps(PROFILES)
-                f.replace_contents(content.encode("utf-8"),
-                                   None,
-                                   False,
-                                   Gio.FileCreateFlags.REPLACE_DESTINATION,
-                                   None)
-                self.__profiles = PROFILES
-        except Exception as e:
-            Logger.error("Application::set_profiles(): %s", e)
-
     def quit(self, vacuum=False):
         """
             Quit application
@@ -246,8 +218,7 @@ class Application(Gtk.Application):
             self.history.clear_to(int(atime))
 
         if self.sync_worker is not None:
-            if self.sync_worker.syncing:
-                self.sync_worker.stop()
+            self.sync_worker.stop()
             self.sync_worker.save_pendings()
         if vacuum:
             task_helper = TaskHelper()
@@ -264,14 +235,6 @@ class Application(Gtk.Application):
         """
         return self.__pinned
 
-    @property
-    def profiles(self):
-        """
-            Get profiles
-            @return {}
-        """
-        return self.__profiles
-
     @property
     def start_page(self):
         """
@@ -359,7 +322,7 @@ class Application(Gtk.Application):
         from eolie.firefox_sync import SyncWorker
         if SyncWorker.check_modules():
             self.sync_worker = SyncWorker()
-            self.sync_worker.sync_loop()
+            self.sync_worker.pull_loop()
         else:
             self.sync_worker = None
         cssProviderFile = Gio.File.new_for_uri(
@@ -413,8 +376,6 @@ class Application(Gtk.Application):
             except:
                 pass
 
-        # Init profiles
-        self.set_profiles()
         # Init default context
         Context(WebKit2.WebContext().get_default())
 
@@ -449,8 +410,8 @@ class Application(Gtk.Application):
         self.set_accels_for_action("win.shortcut::history", ["<Control>h"])
         self.set_accels_for_action("win.shortcut::search", ["<Control>k"])
         self.set_accels_for_action("win.shortcut::fullscreen", ["F11"])
-        self.set_accels_for_action("app.settings",
-                                   ["<Control>s"])
+        self.set_accels_for_action("win.shortcut::settings",
+                                   ["<Control>comma"])
         self.set_accels_for_action("win.shortcut::backward",
                                    ["<Alt>Left", "Back"])
         self.set_accels_for_action("win.shortcut::forward",
@@ -547,7 +508,7 @@ class Application(Gtk.Application):
         """
         try:
             window_states = []
-            for window in self.get_windows():
+            for window in self.windows:
                 window_state = self.__get_state(window)
                 window_states.append(window_state)
             for window_state in self.__state_cache:
@@ -569,13 +530,14 @@ class Application(Gtk.Application):
                 self.__state_cache.remove(state)
                 break
 
-    def __create_initial_windows(self, foreground):
+    def __create_initial_windows(self):
         """
             Create initial windows based on saved session
-            @param foreground  as bool if foreground loading allowed
+            @return active window as Window
         """
         size = (800, 600)
         maximized = False
+        i = 0
         pinned = []
         try:
             pinned = load(
@@ -589,7 +551,6 @@ class Application(Gtk.Application):
                     window.container.sites_manager.set_initial_sort(
                         window_state["sites"])
                     items = []
-                    i = 0 if foreground else 1
                     for (uri, title, atime,
                          ephemeral, state) in window_state["states"]:
                         if uri in pinned:
@@ -614,25 +575,23 @@ class Application(Gtk.Application):
             Logger.error("Application::__create_initial_windows(): %s", e)
         # Create a window if None available and setup pinned pages
         # We already removed saved states from pinned
-        windows = self.get_windows()
-        if windows:
-            window = windows[0]
+        if self.windows:
+            window = self.windows[0]
         else:
             window = self.get_new_window(size, maximized)
-            window.container.add_webview(
-                            self.start_page,
-                            LoadingType.FOREGROUND,
-                            False)
         for uri in pinned:
+            loading_type = wanted_loading_type(i)
             window.container.add_webview(uri,
-                                         LoadingType.BACKGROUND,
+                                         loading_type,
                                          False)
+            i += 1
         # Really setup pinned
         try:
             self.__pinned = load(
                     open(EOLIE_DATA_PATH + "/pinned.bin", "rb"))
         except:
             self.__pinned = []
+        return window
 
     def __on_handle_local_options(self, app, options):
         """
@@ -659,32 +618,31 @@ class Application(Gtk.Application):
         if options.contains("disable-artwork-cache"):
             self.art.disable_cache()
         ephemeral = options.contains("private")
-        if not self.get_windows():
-            self.__create_initial_windows(len(args) < 2)
-            active_window = self.active_window
+
+        active_window = self.active_window
+        # Handle initial windows
+        if not self.windows:
+            active_window = self.__create_initial_windows()
         elif options.contains("new") or len(args) == 1:
             active_window = self.get_new_window()
-        else:
-            active_window = self.active_window
+
         # Open command line args
-        if len(args) > 1:
-            items = []
-            i = 0
-            for uri in args[1:]:
-                # Transform path to uri
-                parsed = urlparse(uri)
-                if not parsed.scheme:
-                    if uri.startswith('/'):
-                        uri = "file://%s" % uri
-                    else:
-                        uri = "http://%s" % uri
-                loading_type = wanted_loading_type(i)
-                items.append((uri, uri, 0, ephemeral, None, loading_type))
-                i += 1
+        items = []
+        i = 0
+        for uri in args[1:]:
+            # Transform path to uri
+            parsed = urlparse(uri)
+            if not parsed.scheme:
+                if uri.startswith('/'):
+                    uri = "file://%s" % uri
+                else:
+                    uri = "http://%s" % uri
+            loading_type = wanted_loading_type(i)
+            items.append((uri, uri, None, ephemeral, None, loading_type))
+            i += 1
+        if items:
             active_window.container.add_webviews(items)
-            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
-            monotonic_time = int(GLib.get_monotonic_time() / 1000)
-            active_window.present_with_time(monotonic_time)
+
         # Add default start page
         if not active_window.container.views:
             active_window.container.add_webview(self.start_page,
@@ -695,6 +653,7 @@ class Application(Gtk.Application):
                                                          self.__on_get_plugins,
                                                          None)
         Gdk.notify_startup_complete()
+        active_window.present()
         return 0
 
     def __close_window(self, window):
@@ -822,9 +781,7 @@ class Application(Gtk.Application):
             @param application as Gio.Application
         """
         if self.get_windows():
-            # https://bugzilla.gnome.org/show_bug.cgi?id=766284
-            monotonic_time = int(GLib.get_monotonic_time() / 1000)
-            self.active_window.present_with_time(monotonic_time)
+            self.active_window.present()
 
     def __on_shortcut_action(self, action, param):
         """
