@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -164,7 +164,7 @@ class SearchEngineDialog:
                 return
 
         item = Item()
-        item.set_property("name",  _("New engine"))
+        item.set_property("name", _("New engine"))
         item.set_property("uri", "")
         item.set_property("search", "")
         item.set_property("bang", "")
@@ -204,7 +204,7 @@ class SearchEngineDialog:
         self.__search_entry.set_text(row.item.get_property("search"))
         self.__bang_entry.set_text(row.item.get_property("bang"))
         default_search_engine = App().settings.get_value(
-                                                  "search-engine").get_string()
+            "search-engine").get_string()
         self.__default_switch.set_active(default_search_engine ==
                                          row.item.get_property("name"))
 
@@ -227,7 +227,7 @@ class SearchEngineDialog:
 
     def _on_uri_entry_changed(self, entry):
         """
-            Update search engine uri
+            Update search engine URI
             @param entry as Gtk.Entry
         """
         uri = entry.get_text()
@@ -243,7 +243,7 @@ class SearchEngineDialog:
 
     def _on_search_entry_changed(self, entry):
         """
-            Update search engine search uri
+            Update search engine search URI
             @param entry as Gtk.Entry
         """
         search = entry.get_text()
