@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # Copyright (c) 2016 Felipe Borges <felipeborges@gnome.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@ from gi.repository import Gio, GLib, WebKit2WebExtension
 from urllib.parse import urlparse
 from uuid import uuid4
 
-from eolie.define import PROXY_BUS, PROXY_PATH, Type
+from eolie.define import PROXY_BUS, PROXY_PATH
 from eolie.list import LinkedList
 from eolie.helper_passwords import PasswordsHelper
 from eolie.logger import Logger
@@ -53,7 +53,7 @@ class Server:
 
         args = list(parameters.unpack())
         for i, sig in enumerate(self.method_inargs[method_name]):
-            if sig is 'h':
+            if sig == 'h':
                 msg = invocation.get_message()
                 fd_list = msg.get_unix_fd_list()
                 args[i] = fd_list.get(args[i])
@@ -202,11 +202,10 @@ class ProxyExtensionServer(Server):
             @param hostname_uri as str
             @param form_uri as str
         """
-        if self.__form_extension.pending_credentials in [None, Type.NONE]:
+        if self.__form_extension.pending_credentials is None:
             return
         try:
-            (_uuid,
-             _user_form_name,
+            (_user_form_name,
              user_form_value,
              _pass_form_name,
              pass_form_value,
@@ -577,6 +576,7 @@ class ProxyExtensionServer(Server):
             @param forms as FormsExtension
             @param variant as GLib.Variant
         """
+        self.__elements_history = {}
         if self.__bus is not None:
             self.__bus.emit_signal(
                 None,
