@@ -1,4 +1,4 @@
-# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2019 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -118,7 +118,9 @@ class PageChildRow(Gtk.ListBoxRow):
         """
         if event in [WebKit2.LoadEvent.STARTED,
                      WebKit2.LoadEvent.COMMITTED]:
-            self.__label.set_text(webview.get_uri())
+            uri = webview.get_uri()
+            if uri is not None:
+                self.__label.set_text(uri)
 
     def __on_webview_title_changed(self, webview, title):
         """
@@ -162,7 +164,7 @@ class SitesManagerChild(Gtk.ListBoxRow):
         builder.add_from_resource("/org/gnome/Eolie/SitesManagerChild.ui")
         builder.connect_signals(self)
         widget = builder.get_object("widget")
-        self.__indicator_label = LabelIndicator()
+        self.__indicator_label = LabelIndicator(True)
         self.__indicator_label.set_property("halign", Gtk.Align.CENTER)
         self.__indicator_label.show()
         self.__separator = builder.get_object("separator")
@@ -193,7 +195,11 @@ class SitesManagerChild(Gtk.ListBoxRow):
         if view not in self.__views:
             self.__views.append(view)
             view.webview.connect("shown", self.__on_webview_shown)
-            view.webview.connect("favicon-changed",
+            view.webview.connect("load-changed",
+                                 self.__on_webview_load_changed)
+            view.webview.connect("notify::is-playing-audio",
+                                 self.__on_webview_notify_is_playing_audio)
+            view.webview.connect("notify::favicon",
                                  self.__on_webview_favicon_changed)
             self.update_label()
             self.__indicator_label.update_count(True)
@@ -215,6 +221,9 @@ class SitesManagerChild(Gtk.ListBoxRow):
             self.__views.remove(view)
             view.webview.disconnect_by_func(self.__on_webview_shown)
             view.webview.disconnect_by_func(self.__on_webview_favicon_changed)
+            view.webview.disconnect_by_func(self.__on_webview_load_changed)
+            view.webview.disconnect_by_func(
+                                    self.__on_webview_notify_is_playing_audio)
             self.update_label()
             self.__indicator_label.update_count(False)
             if not view.webview.shown:
@@ -296,10 +305,10 @@ class SitesManagerChild(Gtk.ListBoxRow):
                     if child.view == current:
                         self.__pages_listbox.select_row(child)
                         self.__current_child = child
+                self.__pages_listbox.invalidate_sort()
             else:
                 self.__pages_listbox.unselect_all()
                 self.__current_child = None
-            self.__pages_listbox.invalidate_sort()
 
     @property
     def empty(self):
@@ -349,21 +358,14 @@ class SitesManagerChild(Gtk.ListBoxRow):
                 return True
             elif event.button == 3:
                 from eolie.menu_sites import SitesMenu
-                from eolie.menu_profiles import ProfilesMenu
                 from eolie.menu_move_to import MoveToMenu
                 sites_menu = SitesMenu(self.__views, self.__window)
                 sites_menu.show()
-                webview = self.__views[0].webview
-                profiles_menu = ProfilesMenu(webview, self.__window)
-                profiles_menu.show()
                 moveto_menu = MoveToMenu(self.__views, self.__window)
                 moveto_menu.show()
                 popover = Gtk.PopoverMenu.new()
                 popover.add(sites_menu)
-                popover.add(profiles_menu)
                 popover.add(moveto_menu)
-                popover.child_set_property(profiles_menu,
-                                           "submenu", "profiles")
                 popover.child_set_property(moveto_menu,
                                            "submenu", "moveto")
                 popover.set_relative_to(eventbox)
@@ -404,37 +406,38 @@ class SitesManagerChild(Gtk.ListBoxRow):
         elif hasattr(widget, "forall"):
             GLib.idle_add(widget.forall, self.__update_popover_internals)
 
-    def __on_webview_favicon_changed(self, webview, surface=None):
+    def __on_webview_notify_is_playing_audio(self, webview, playing):
+        """
+            Update favicon
+            @param webview as WebView
+            @param playing as bool
+        """
+        self.__on_webview_favicon_changed(webview)
+
+    def __on_webview_favicon_changed(self, webview, *ignore):
         """
             Set favicon
             @param webview as WebView
-            @param surface as cairo.Surface
         """
-        if len(self.__views) > 1 and\
-                webview.view != self.__window.container.current:
-            return
-        # Try to get it from webview, because cached one may not
-        # be identical
-        if surface is None:
-            surface = webview.get_favicon()
-        if surface is not None:
-            self.__image.set_from_surface(resize_favicon(surface))
+        if webview.current_event == WebKit2.LoadEvent.STARTED:
             return
 
-        favicon_path = App().art.get_favicon_path(webview.uri)
-        if favicon_path is not None:
-            self.__image.set_from_file(favicon_path)
+        if webview.is_playing_audio():
+            self.__image.set_from_icon_name("audio-speakers-symbolic",
+                                            Gtk.IconSize.INVALID)
             return
 
         artwork = App().art.get_icon_theme_artwork(webview.uri,
                                                    webview.ephemeral)
         if artwork is not None:
-            self.__image.set_from_icon_name(artwork,
-                                            Gtk.IconSize.INVALID)
-            return
-
-        self.__image.set_from_icon_name("applications-internet",
-                                        Gtk.IconSize.INVALID)
+            self.__image.set_from_icon_name(artwork, Gtk.IconSize.INVALID)
+        else:
+            surface = webview.get_favicon()
+            if surface is not None:
+                self.__image.set_from_surface(resize_favicon(surface))
+            else:
+                self.__image.set_from_icon_name("applications-internet",
+                                                Gtk.IconSize.INVALID)
 
     def __on_query_tooltip(self, widget, x, y, keyboard, tooltip):
         """
@@ -537,3 +540,15 @@ class SitesManagerChild(Gtk.ListBoxRow):
         """
         self.__indicator_label.mark_shown(webview)
         self.__on_webview_favicon_changed(webview)
+
+    def __on_webview_load_changed(self, webview, event):
+        """
+            Update widget content
+            @param webview as WebView
+            @param event as WebKit2.LoadEvent
+        """
+        if event == WebKit2.LoadEvent.STARTED:
+            self.__image.set_from_icon_name("content-loading-symbolic",
+                                            Gtk.IconSize.INVALID)
+        elif event == WebKit2.LoadEvent.FINISHED:
+            self.__on_webview_favicon_changed(webview)
